import { readingTime } from 'reading-time-estimator';
import path from 'path';
import { readFile } from 'fs/promises';
import { slugify } from '$lib/slugify.js';
import _ from 'lodash';

export async function load() {
	const allPostFiles = import.meta.glob<SvelteAllProps>('/src/routes/posts/*.md');
	const iterablePostFiles = Object.entries(allPostFiles);

	const markdownFilesPath = path.resolve(
		process.env.npm_package_json ? path.dirname(process.env.npm_package_json) : '.',
		'src',
		'routes',
		'posts',
	);

	const posts = await Promise.all(
		iterablePostFiles.map(async ([filePath, resolver]) => {
			const { metadata } = await resolver();

			const slug = slugify(filePath);
			const contents = await readFile(path.join(markdownFilesPath, `${slug}.md`), 'utf8');

			return {
				meta: metadata,
				path: slug,
				readingTime: readingTime(contents),
			};
		}),
	);

	const mostRecentPost = _.maxBy(posts, ({ meta }) => meta.date);
	const highlightedPosts = _.reverse(
		_.sortBy(
			posts.filter(
				(post) => post.meta.highlighted && (!mostRecentPost || post.path !== mostRecentPost.path),
			),
			({ meta }) => meta.date,
		),
	).slice(0, 4);

	return { highlightedPosts, mostRecentPost };
}
